############################################################################################################
# Network analysis
# Figures 3
# Supp Fig S3 (A, B)
############################################################################################################
# get a network of KEGG pathways (with size > 10)
which.large = sapply(genes.by.pathway, length)>10
ids = names(genes.by.pathway[which.large])

# make the pathway names shorter
shortids = sapply(strsplit(ids, split="hsa"), "[[", 2)
shortids = as.character(as.integer(shortids))
names(shortids) = ids

# net info
keggpairs = t(combn(shortids,2))
overlap = c()
for(i in 1:nrow(keggpairs)) {
  id1 = names(which(shortids==keggpairs[i,1]))
  id2 = names(which(shortids==keggpairs[i,2]))
  genes1 = genes.by.pathway[[id1]]
  genes2 = genes.by.pathway[[id2]]
  common = intersect(genes1, genes2)
  both = union(genes1, genes2)
  overlap[i] = length(common)/length(both)
}
netdat = data.frame(keggpairs, overlap)
colnames(netdat) = c("from","to","Overlap")

# keep those kegg pairs with overlap above a threshold (5%)
overlap.threshold = 0.05
links = netdat[netdat[,3]>overlap.threshold,]
nodes = unique(as.character(unlist(links[,1:2])))


# create a graph object
require(igraph)
g <- graph_from_data_frame(d=links, vertices=nodes, directed=F)
# plot(g, vertex.label.color="black", vertex.label.dist=1, vertex.size=7)
E(g)$width = links[,3]

# Identify vertices with coresness <= threshold
toosmall <- which(graph.coreness(g) <= 2)
# Delete vertices and plot
gp <- delete.vertices(g, toosmall)

rm(ids, id1, id2, genes1, genes2, common, both, overlap, netdat, overlap.threshold, links, nodes, toosmall)
deg_dist_gp <- degree_distribution(gp, cumulative = TRUE)

#####################################################################
# Supplementary Figure S3_A
#####################################################################
graphics.off()
par(mfrow = c(1, 2), pty="s")
plot(deg_dist_gp, xlab="Degree", ylab="Cumulative Frequency", pch=16, cex=0.86, 
     main="Degree distribution of \n KEGG pathway overlap network")

figs3a = "Figures_for_manuscript/supp_figs/Fig_S3A.pdf"
if(!file.exists(figs3a)) {
  dev.copy(device=pdf, file=figs3a)
  dev.off()
} else {
  print.noquote(paste0("Figure exists - ", figs3a))
}
rm(figs3a)

#####################################################################
# Supplementary Figure S3_B
#####################################################################
## Boxplot of degrees of selected and other pathways
dg = degree(gp)
names(dg) = paste0("hsa",formatC(as.integer(names(dg)), width=5, flag="0"))
dg.sel = dg[sel]
dg.other = dg[setdiff(names(dg),sel)]
boxplot(dg.sel, dg.other, col=c("red", "gray90"), 
        main="Box plot showing Centrality\nof the Selected Pathways",
        names=c("Selected 66 pathways", "Other pathways"),
        ylab="Degree of the node in the KEGG overlap network")
cat("Degree distribution of the nodes in the KEGG network: ")

figs3b = "Figures_for_manuscript/supp_figs/Fig_S3B.pdf"
if(!file.exists(figs3b)) {
  dev.copy(device=pdf, file=figs3b)
  dev.off()
} else {
  print.noquote(paste0("Figure exists - ", figs3b))
}
rm(figs3b)

#####################################################################
# Figure 3
#####################################################################

# layout
coords = layout_(gp, nicely())

# color the nodes on the network
sel.nodes = c(pathways.c.only, pathways.sl.only, pathways.both)
nodes.66 = shortids[sel.nodes]

nodenames = vertex_attr(gp)$name
mycolors = rep("gray90",length(nodenames))
names(mycolors) = nodenames

mycolors[nodenames%in%nodes.66] = "red"
# resize the vertex size
node.size <- rep(4, length(V(gp)))

V(gp)$color = mycolors

 
scan()
graphics.off()
#opar = par()
par(mar=c(1,1,1,1))
plot(gp, layout=coords, vertex.label.color=V(gp)$color, vertex.label.dist=1, vertex.label=NA,
     vertex.label.cex=0.8, vertex.label.dist=1, 
     vertex.size= node.size, vertex.frame.color=0, edge.curved=0.5)
#par(opar)

#tkid <- tkplot(gp) #tkid is the id of the tkplot that will open
#l <- tkplot.getcoords(tkid) # grab the coordinates from tkplot
#plot(net, layout=l)

#rm(all.node.dat, sel.node.dat, keggnames, node.size, mycolors, nodenames, shortids, g, gp, i)
cat("Selected pathways are at the core of the network!")

