# This code performs analysis of sepsis and cancer datasets 
# Samanwoy Mukhopadhyay
# 28 Feb 2019
# 02 May 2019
# Saroj Kant Mohapatra
# Revised for re-submission to BMC Cancer
# Oct-Nov 2019

# clean up work area
rm(list=ls()); graphics.off()
###############################
# Set Data directory
###############################
#showPlots = TRUE
sig.threshold <- 0.01

# Use the packages for loading data
###################################
library("ssnibmg")
library("tcnibmg")
# Load the data
################
data("ss.eset")
data("tc.eset")
###############################
# preliminaries
source("Rcode/prelims.R") # Loads the data from Mukho2017
###############################
# get data
source("Rcode/getData.R")
###############################

##############################
# Hierarchical clustering of diseases using 90 pathways
# Fig S1: dendrogram_90_pathways
# Get the data for selected pathways
sstcga = gsea.list$Zobs[sel.pathways,]
d = dist(t(sstcga))
hc = hclust(d)
plot(hc, hang=-0.5, axes=F, xlab="", ylab="", main="Hierarchical clustering of diseases\n (with 90 pathways)",
     sub="")
title(sub="Supplementary Figure S1\
For the 90 pathways significantly associated with SS, pathway scores\
were computed for the cancer studies. Scores for SS (6 studies) and cancer\
(17 studies) were subjected to hierarchical clustering (details in main text).\
As shown in this dendrogram, there is clear segregation of the cancer studies\
into two groups: sepsis-like (SLC) and other cancer (CA).", adj=0)
figs1 = "Figures_for_manuscript/supp_figs/Fig_S1.pdf"
if(!file.exists(figs1)) {
  dev.copy(device=pdf, file=figs1)
  dev.off()
} else {
  print.noquote(paste0("Figure exists - ", figs1))
}
rm(d, figs1)

#cat("Hierarchical clustering of studies: ");scan("")

###############################
# Find the groups of cancer in the sepsis cluster
# Fig 2A: heatmap (of 66 pathways by all studies)

###############################
fig2a = "Figures_for_manuscript/Fig_2A.pdf"
if(!file.exists(file=fig2a)) {
  pdf(file=fig2a)
  source("Rcode/drawPathwayHeatmap.R")
  dev.off()  
} else {
  print.noquote(paste0("Figure exists - ", fig2a))
  source("Rcode/drawPathwayHeatmap.R")
}
rm(fig2a)

#cat("Pathway score heatmap: ");scan("")

###############################
# Fig 2B
# Draw a box plot of mean pathway score for the three disease groups
# for the following pathways
# (1) all 66, (2) CA-only, (3) SLC-only, (4) both CA and SLC 
###############################
# get the pathway scores for 66 pathways
# This shows up-regulation in SLC as (with greater magnitude) SS
# and down-regulation in CA
fig2b = "Figures_for_manuscript/Fig_2B.pdf"
if(!file.exists(file=fig2b)) {
  pdf(file=fig2b)
  drawBoxplot3DiseaseGroup()
  dev.off()
} else {
  print.noquote(paste0("Figure exists - ", fig2b))
  drawBoxplot3DiseaseGroup()
}
rm(fig2b)

#####################################
# sorting disease from highest number of pertubed pathways (66 in SS)
# to smallest number
sort(apply(gsea.list$pvals[sel,]<0.01, 2, sum), dec=TRUE)

############################################################################################################
# PCA analysis
# Fig: shows three different groups
# The following few lines 
############################################################################################################
# drawing PCA for Cancers and SS
# pca = prcomp(sstcga[sel, ids])
# pca3d(pca$rotation, col=sidecols, radius=3)
# legend3d("topright", legend = paste( c("SS", "SLC", "CA")), pch=19, col = c("magenta", "cyan", "blue"),
#          cex=1.2, pt.cex=2, inset=c(0.06, 0.06), bty="n")

# making a gif for PCA
# makeMoviePCA(dir="~/Desktop", clean=TRUE, type = "gif",  movie = "PCAmovie", convert=TRUE)
# rm(ids, sidecols)
############################################################################################################
# Network analysis
# Fig 3   Degree distribution and box plots
# Fig 4   Network diagram
############################################################################################################
source("Rcode/net.R")
graphics.off()
############################################################################################################
# Draw individual Pathway gene heatmaps
# Fig: S2
############################################################################################################
# Setting pdf height 
# hsa04914 - 10

figs2 = "Figures_for_manuscript/supp_figs/Fig_S2.pdf"
if(!file.exists(figs2)) {
  fcounter = 1
  p66ann = read.table(file="Metadata/pathways66annot.csv", header=TRUE, sep=",")
  rownames(p66ann) = p66ann$Pathway
  pathway.groups = as.character(sort(unique(p66ann[,3])))
  for(grp in pathway.groups) {
    pathids = as.character(p66ann[grp==p66ann[,3],1])
    for(pathid in pathids) {
      print(pathid)
      #x11(width=5, height=20)
      pdf(height=p66ann[pathid,"figheight"], 
          file=paste0("f",fcounter,".pdf"))
      fcounter=fcounter+1
      drawPathwayGeneHeatmap(pathid, groupid=grp)
      dev.off()
    }
  }
  commandstr = paste0("gs -dBATCH -dNOPAUSE -q -sDEVICE=pdfwrite -sOutputFile=", figs2, " ")
  for(i in 1:66) {
    fn = paste0("f", i, ".pdf ")
    commandstr = paste0(commandstr, fn)   
    system(paste0("rm ", fn))
  }
  system(commandstr)
  rm(grp, pathid, p66ann, commandstr)
} else {
  print.noquote(paste0("Figure exists - ", figs2))
}
rm(figs2)

#####################################################################
# Table 2
# Machine learning (SVM and NN) - based Classification of samples
# based on 66 pathway expression
#####################################################################
# Prepare data for machine learning
source("Rcode/prepareData4Mlearn.R")

# function to get misclassification rate
getMiss = function(cmatrix) {
  100*sum(cmatrix[2]+cmatrix[3])/sum(cmatrix)
}

# Support Vector Machine: 10-fold cross validation
library(MLInterfaces)
set.seed(1234)
# xvsv = MLearn(label~., pathES, svmI, xvalSpec("LOG", 10, balKfold.xvspec(10)))
# SVM models stored
# Results/mlearn.models/xvsv.MR9.15.rda - 10-fold Misclassification rate 9.15% 
load("Results/mlearn.models/xvsv.MR9.15.rda")
cmatrix = confuMat(xvsv)
cmatrix
print(paste0("Misclassification rate for SVM: ", formatC(getMiss(cmatrix), 2), "%"))

# Neural Net: 10-fold cross validation
set.seed(1234)
# xvnn = MLearn(label~., pathES, nnetI, xvalSpec("LOG", 10, balKfold.xvspec(10)),
#                  size=3, decay=0.5)
# NN models stored
# Results/mlearn.models/xvnn.MR8.68.rda - 10-fold Misclassification rate 8.06% 
load("Results/mlearn.models/xvnn.MR8.06.rda")
cmatrix = confuMat(xvnn)
cmatrix
print(paste0("Misclassification rate for NN: ", formatC(getMiss(cmatrix), 2), "%"))

###############################
# Supplementary Figure 4
# Survival analysis
# For each of the selected pathways
# do survival analysis and save the p-values 
# in a matrix
###############################

fn_tableS3 = "Results/pathbycancertypeSurvPval.csv"
if(file.exists(file=fn_tableS3)) {
  path66ByProj = read.table(file=fn_tableS3, header=TRUE, sep="\t")
} else {
  proj.ids = c("HNSC","KIRC","LIHC","BRCA","LUAD","LUSC","PRAD","THCA")
  path66ByProj = matrix(NA, nrow=length(sel), ncol=length(proj.ids))
  colnames(path66ByProj) = proj.ids
  rownames(path66ByProj) = sel
  
  for(pathid in rownames(path66ByProj)) {
    pathstr = pathways.list[paste0("path:",pathid)]
    pathstr = strsplit(pathstr, split=" -")[[1]][1]
    pathstr = paste0("Pathway: ", pathstr, " [", pathid, "]")
    #graphics.off(); rm(sfit, dat, info, pathX, projES, to.keep, sel.ptids)
    for(proj in colnames(path66ByProj)) {
      graphics.off(); rm(sfit, dat, info, pathX, projES, to.keep, sel.ptids)
      # get the clinical metadata from an external file and add to the expression set
      info = read.table(file=paste0("Metadata/",proj,"_surv_clinical.tsv"), header=T, sep="\t")
      to.keep = (apply(info[,2:3], 1, function(x) !all(x=="--")))
      info = info[to.keep,]
      is.na(info) <- info=="--"
      
      rownames(info) = gsub("-",".", info$submitter_id)
      
      sel.ptids = intersect(rownames(info), sampleNames(pathES))
      #print.noquote(length(sel.ptids))
      projES = pathES[, sel.ptids]
      info = info[sel.ptids,]
      
      pathX = as.vector(exprs(projES[paste0("x.", pathid), ]))
      names(pathX) = sampleNames(projES)
      dat = data.frame(info, pathX)
      dat$day = as.integer(as.character(dat$days_to_death))
      dat$day[dat$outcome==0] = as.integer(as.character(dat$days_to_last_follow_up[dat$outcome==0]))
      dat$high_expressing = as.integer(dat$pathX > median(dat$pathX))
      dat = dat[!is.na(dat$day),]
      #print.noquote(nrow(dat))
      sfit <- survfit(Surv(day, outcome)~high_expressing, data=dat)
      res = ggsurvplot(sfit, conf.int=FALSE, pval=TRUE, risk.table=TRUE, 
                       legend.labs=c("Low", "High"), 
                       legend.title=paste0(pathid, " Score"),  
                       palette=c("dodgerblue2", "orchid2"), 
                       title=paste0("Kaplan-Meier Curve for ", proj, " Survival\n",pathstr), 
                       risk.table.height=.25)
      pval = surv_pvalue(sfit, data = NULL, method = "survdiff",
                         test.for.trend = FALSE, combine = FALSE)$pval
      cat(paste0(pathid, " ", proj, " ", formatC(pval, digits=2), "\n"))
      pathidprint(res)
      path66ByProj[pathid, proj] = pval
    }
  }
  
  selnames = sapply(strsplit(pathways.list[paste0("path:",sel)], split=" -"), function(x) x[[1]][1]) %>% as.character
  rownames(path66ByProj) = paste0(selnames, " | ", sel)
  write.table(path66ByProj, file="Results/pathbycancertypeSurvPval.csv", row.names=TRUE, col.names = TRUE, quote=F, sep="\t")
}

ids.slc = intersect(colnames(path66ByProj), ids.sepsis.like)
ids.ca  = intersect(colnames(path66ByProj), ids.cancer)
percSignifSurv = apply(path66ByProj, 2, function(x) 100*sum(x<0.1)/length(x))
mycols = c(rep("blue",length(ids.ca)), rep("cyan",length(ids.slc)))
names(mycols) = c(ids.ca, ids.slc)
percSignifSurv = percSignifSurv[c(ids.ca, ids.slc)]

# Figure 4: survival analysis and K-M plot
source("Rcode/drawSurvPlots.R")



# Send the following text to legend for Figure 4
# mtext("Supplementary Figure S3\
# Eight cancer types (5 CA and 3 SLC) were selected for survival analysis.\
# For each cancer, association with survival was accepted at threshold of\
# p < 0.1. For each cancer type, proportion of pathways associated with\
# survival was calculated and converted to percentage. The percentage\
# values are presented in this barplot. As shown for the selected pathways,\
# association with survival is generally higher for the SLC group than\
# for CA group cancer types.", line=10, side=1, adj=0)

figs4 = "Figures_for_manuscript/Fig_4.pdf"
if(!file.exists(figs4)) {
  dev.copy(device=pdf, file=figs4)
  dev.off()
} else {
  print.noquote(paste0("Figure exists - ", figs4))
}
#rm(percSignifSurv, ids.slc, ids.ca, mycols, figs3, fn_tableS3)



